@@ -207,7 +207,7 @@ class _IsScoredDescriptor(SonolusDescriptor):
         if instance is None:
             return self.value
         elif ctx():
-            raise RuntimeError("Cannot access is_scored from self in a callback, use ArchetypeClass.is_scored")
+            return ctx().global_state.is_scored_by_archetype_id[instance.id]
         else:
             return self.value
 
@@ -237,7 +237,7 @@ class _KeyDescriptor(SonolusDescriptor):
 
     def __get__(self, instance, owner):
         if instance is not None and ctx():
-            return ctx().global_state.keys[instance.id]
+            return ctx().global_state.keys_by_archetype_id[instance.id]
         else:
             return self.value
 
@@ -245,6 +245,21 @@ class _KeyDescriptor(SonolusDescriptor):
         raise AttributeError("Archetype key is read-only and cannot be set")
 
 
+class _ArchetypeLifeDescriptor(SonolusDescriptor):
+    def __get__(self, instance, owner):
+        if not ctx():
+            raise RuntimeError("Archetype life is only available during compilation")
+        if ctx().global_state.mode not in {Mode.PLAY, Mode.WATCH}:
+            raise RuntimeError(f"Archetype life is not available in mode '{ctx().global_state.mode.value}'")
+        if instance is not None:
+            return _deref(ctx().blocks.ArchetypeLife, instance.id * ArchetypeLife._size_(), ArchetypeLife)
+        else:
+            return _deref(ctx().blocks.ArchetypeLife, owner.id * ArchetypeLife._size_(), ArchetypeLife)
+
+    def __set__(self, instance, value):
+        raise AttributeError("Archetype life is read-only and cannot be set")
+
+
 def imported(*, name: str | None = None) -> Any:
     """Declare a field as imported.
 
@@ -589,7 +604,9 @@ class _BaseArchetype:
         cls._key_ = cls.key
         cls.key = _KeyDescriptor(cls.key)
         cls.name = _NameDescriptor(cls.name)
+        cls._is_scored_ = cls.is_scored
         cls.is_scored = _IsScoredDescriptor(cls.is_scored)
+        cls.life = _ArchetypeLifeDescriptor()
 
     @classmethod
     def _init_fields(cls):
@@ -605,8 +622,10 @@ class _BaseArchetype:
                 "id",
                 "key",
                 "name",
+                "life",
                 "is_scored",
                 "_key_",
+                "_is_scored_",
                 "_derived_base_",
                 "_is_derived_",
                 "_default_callbacks_",
@@ -809,7 +828,10 @@ class PlayArchetype(_BaseArchetype):
     _supported_callbacks_ = PLAY_CALLBACKS
 
     is_scored: ClassVar[bool] = False
-    """Whether the entity contributes to combo and score."""
+    """Whether entities of this archetype contribute to combo and score."""
+
+    life: ClassVar[ArchetypeLife]
+    """How this entities of this archetype contribute to life depending on judgment."""
 
     def preprocess(self):
         """Perform upfront processing.
@@ -925,18 +947,6 @@ class PlayArchetype(_BaseArchetype):
         """Whether this entity is despawned."""
         return self._info.state == 2
 
-    @property
-    @meta_fn
-    def life(self) -> ArchetypeLife:
-        """How this entity contributes to life."""
-        if not ctx():
-            raise RuntimeError("Calling life is only allowed within a callback")
-        match self._data_:
-            case _ArchetypeSelfData() | _ArchetypeReferenceData():
-                return _deref(ctx().blocks.ArchetypeLife, self.id * ArchetypeLife._size_(), ArchetypeLife)
-            case _:
-                raise RuntimeError("Life is not available in level data")
-
     @property
     @meta_fn
     def result(self) -> PlayEntityInput:
@@ -952,26 +962,6 @@ class PlayArchetype(_BaseArchetype):
             case _:
                 raise RuntimeError("Result is only accessible from the entity itself")
 
-    @classmethod
-    def update_life(
-        cls,
-        perfect_increment: int | None = None,
-        great_increment: int | None = None,
-        good_increment: int | None = None,
-        miss_increment: int | None = None,
-    ):
-        """Update the life of this archetype.
-
-        Values default to 0.
-
-        Args:
-            perfect_increment: The increment for perfect life.
-            great_increment: The increment for great life.
-            good_increment: The increment for good life.
-            miss_increment: The increment for miss life.
-        """
-        archetype_life_of(cls).update(perfect_increment, great_increment, good_increment, miss_increment)
-
 
 class WatchArchetype(_BaseArchetype):
     """Base class for watch mode archetypes.
@@ -993,6 +983,12 @@ class WatchArchetype(_BaseArchetype):
 
     _supported_callbacks_ = WATCH_ARCHETYPE_CALLBACKS
 
+    is_scored: ClassVar[bool] = False
+    """Whether entities of this archetype contribute to combo and score."""
+
+    life: ClassVar[ArchetypeLife]
+    """How this entities of this archetype contribute to life depending on judgment."""
+
     def preprocess(self):
         """Perform upfront processing.
 
@@ -1058,18 +1054,6 @@ class WatchArchetype(_BaseArchetype):
         """Whether this entity is active."""
         return self._info.state == 1
 
-    @property
-    @meta_fn
-    def life(self) -> ArchetypeLife:
-        """How this entity contributes to life."""
-        if not ctx():
-            raise RuntimeError("Calling life is only allowed within a callback")
-        match self._data_:
-            case _ArchetypeSelfData() | _ArchetypeReferenceData():
-                return _deref(ctx().blocks.ArchetypeLife, self.id * ArchetypeLife._size_(), ArchetypeLife)
-            case _:
-                raise RuntimeError("Life is not available in level data")
-
     @property
     @meta_fn
     def result(self) -> WatchEntityInput:
@@ -1090,26 +1074,6 @@ class WatchArchetype(_BaseArchetype):
         if cls._exported_fields_:
             raise RuntimeError("Watch archetypes cannot have exported fields")
 
-    @classmethod
-    def update_life(
-        cls,
-        perfect_increment: int | None = None,
-        great_increment: int | None = None,
-        good_increment: int | None = None,
-        miss_increment: int | None = None,
-    ):
-        """Update the life of this archetype.
-
-        Values default to 0.
-
-        Args:
-            perfect_increment: The increment for perfect life.
-            great_increment: The increment for great life.
-            good_increment: The increment for good life.
-            miss_increment: The increment for miss life.
-        """
-        archetype_life_of(cls).update(perfect_increment, great_increment, good_increment, miss_increment)
-
 
 class PreviewArchetype(_BaseArchetype):
     """Base class for preview mode archetypes.
@@ -1167,6 +1131,23 @@ class PreviewArchetype(_BaseArchetype):
             raise RuntimeError("Preview archetypes cannot have exported fields")
 
 
+@meta_fn
+def get_archetype_by_name(name: str) -> AnyArchetype:
+    """Return the archetype with the given name in the current mode."""
+    if not ctx():
+        raise RuntimeError("Archetypes by name are only available during compilation.")
+    name = validate_value(name)
+    if not name._is_py_():
+        raise TypeError(f"Invalid name: '{name}'")
+    name = name._as_py_()
+    if not isinstance(name, str):
+        raise TypeError(f"Invalid name: '{name}'")
+    archetypes_by_name = ctx().global_state.archetypes_by_name
+    if name not in archetypes_by_name:
+        raise KeyError(f"Unknown archetype: '{name}'")
+    return archetypes_by_name[name]
+
+
 @meta_fn
 def entity_info_at(index: int) -> PlayEntityInfo | WatchEntityInfo | PreviewEntityInfo:
     """Retrieve entity info of the entity at the given index.
@@ -1186,23 +1167,6 @@ def entity_info_at(index: int) -> PlayEntityInfo | WatchEntityInfo | PreviewEnti
             raise RuntimeError(f"Entity info is not available in mode '{ctx().global_state.mode}'")
 
 
-@meta_fn
-def archetype_life_of(archetype: type[_BaseArchetype] | _BaseArchetype) -> ArchetypeLife:
-    """Retrieve the archetype life of the given archetype.
-
-    Available in play and watch mode.
-    """
-    archetype = validate_value(archetype)  # type: ignore
-    archetype = archetype._as_py_()  # type: ignore
-    if not ctx():
-        raise RuntimeError("Calling archetype_life_of is only allowed within a callback")
-    match ctx().global_state.mode:
-        case Mode.PLAY | Mode.WATCH:
-            return _deref(ctx().blocks.ArchetypeLife, archetype.id * ArchetypeLife._size_(), ArchetypeLife)
-        case _:
-            raise RuntimeError(f"Archetype life is not available in mode '{ctx().global_state.mode}'")
-
-
 class PlayEntityInfo(Record):
     index: int
     archetype_id: int
