@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from typing import Self
+
 from sonolus.backend.visitor import compile_and_call
 from sonolus.script.array import Array
 from sonolus.script.array_like import ArrayLike, get_positive_index
@@ -102,7 +104,7 @@ class VarArray[T, Capacity](Record, ArrayLike[T]):
     _array: Array[T, Capacity]
 
     @classmethod
-    def new(cls):
+    def new(cls) -> Self:
         """Create a new empty array."""
         element_type = cls.type_var_value(T)
         capacity = cls.type_var_value(Capacity)
@@ -380,7 +382,7 @@ class ArraySet[T, Capacity](Record):
     _values: VarArray[T, Capacity]
 
     @classmethod
-    def new(cls):
+    def new(cls) -> Self:
         """Create a new empty set."""
         element_type = cls.type_var_value(T)
         capacity = cls.type_var_value(Capacity)
@@ -394,7 +396,7 @@ class ArraySet[T, Capacity](Record):
         """Return whether the given value is present in the set."""
         return value in self._values
 
-    def __iter__(self):
+    def __iter__(self) -> SonolusIterator[T]:
         """Return an iterator over the values in the set."""
         return self._values.__iter__()
 
@@ -457,7 +459,7 @@ class ArrayMap[K, V, Capacity](Record):
     _array: Array[_ArrayMapEntry[K, V], Capacity]
 
     @classmethod
-    def new(cls):
+    def new(cls) -> Self:
         """Create a new empty map."""
         key_type = cls.type_var_value(K)
         value_type = cls.type_var_value(V)
@@ -489,7 +491,7 @@ class ArrayMap[K, V, Capacity](Record):
         """Return an iterator over the key-value pairs in the map."""
         return _ArrayMapEntryIterator(self, 0)
 
-    def __iter__(self):
+    def __iter__(self) -> SonolusIterator[K]:
         """Return an iterator over the keys in the map."""
         return self.keys()
 
