@@ -21,6 +21,7 @@ class Effect(Record):
     id: int
     """Effect ID."""
 
+    @property
     def is_available(self) -> bool:
         """Return whether the effect clip is available."""
         return _has_effect_clip(self.id)
@@ -139,7 +140,7 @@ def effect(name: str) -> Any:
 type Effects = NewType("Effects", Any)  # type: ignore
 
 
-@dataclass_transform()
+@dataclass_transform(kw_only_default=True)
 def effects[T](cls: type[T]) -> T | Effects:
     """Decorator to define effect clips.
 
