@@ -1,3 +1,4 @@
+from sonolus.script.array import Dim
 from sonolus.script.containers import ArraySet
 from sonolus.script.stream import Stream, StreamGroup, streams
 
@@ -5,8 +6,8 @@ from sonolus.script.stream import Stream, StreamGroup, streams
 @streams
 class Streams:
     # Records the set of lanes at each time when the empty tap lane effect was played.
-    effect_lanes: Stream[ArraySet[float, 16]]
+    effect_lanes: Stream[ArraySet[float, Dim[16]]]
 
     # Records whether a hold is active at a given time.
     # Keyed by the hold head's index.
-    hold_activity: StreamGroup[bool, 99999]
+    hold_activity: StreamGroup[bool, Dim[99999]]
